#!/usr/bin/perl -w

use strict;
use warnings;

use Date::Tiny;
use Person;
use FamilyTreeData;

package SimonWardFamilyTreeDataFactory;

sub getFullName {
  my ($id) = @_;
  $id =~ s/[0-9]//g;
  return $id;
}

sub getID {
  my ($name_with_space) = @_;
  $name_with_space =~ s/ /_/g;
  return $name_with_space;
}

sub setName {
  my ( $person, $full_name ) = @_;
  my @full_name_a = split( / /, getFullName( $full_name ) );
  $person->{first_name} = $full_name_a[0];
  $person->{last_name}  = $full_name_a[-1];
  $person->{mid_name} = join( ' ', @full_name_a[ 1 .. $#full_name_a - 1 ] )
    if(@full_name_a > 2);  
}
sub validIDEntry {
	my ( $entry ) = @_;
	return defined $entry && $entry !~ m/^\s*$/;
}
sub validEmail {
	my ( $entry ) = @_;
	return defined $entry && $entry =~ m/(\w[-._\w]*\w@\w[-._\w]*\w\.\w{2,3})/;
}
sub validURL {
	my ( $entry ) = @_;
	return defined $entry && $entry =~ m/^http:.*/; 	

}
sub validYear {
  my ($year) = @_;
  return $year >=0 && $year <= `date '+%Y'`;  
}
sub validMonth {
  my ($month) = @_;
  return $month >=1 && $month <= 12;
  
}
sub validGender {
  my ($gender) = @_;
  return defined $gender && ($gender==0 || $gender==1);
  
}
sub getDate {
   my ($date) = @_;
   my @date_a = split( /\//, $date );
   if(scalar @date_a == 1) {
     if(validYear($date_a[0])) { #speed up needed!! 
      return Date::Tiny->new(year  => $date_a[0]);
     }
     else {
      print "Erroneous year:$date\n";
     }
   }
   elsif(scalar @date_a == 2) {
     return Date::Tiny->new(month => $date_a[0],
        year  => $date_a[1]);
   }
   elsif(scalar @date_a == 3) {
     return Date::Tiny->new(day => $date_a[0],
        month => $date_a[1],
        year  => $date_a[2]);
   }
   else {
     print "Erroneous date:$date\n";
   }
}
sub fill_up_pictures {
  my($family_tree_data, $picture_directory) = @_;
  while ( my ($id, $person) = each %{$family_tree_data->{people}}) {
      $person->{picture_file} = getPictureFileName(getFileName($id), $picture_directory) 
  }
  
}
sub getPictureFileName{
	my($id, $picture_directory) = @_;			#speed-up: remove parameter passing
	
	if(-e "$picture_directory/$id.jpg" ) {
		return "$id.jpg";
	}
	elsif(-e "$picture_directory/$id.gif" ) {
		return "$id.gif";
	}
	elsif(-e "$picture_directory/$id.png" ) {
		return "$id.png"; {
    
  }
	}
	else {
		return undef;
	}
	
}
sub set_parent {
  my ($field, $family_tree_data, $temp_person, $parent_type, $parent_type_code) = @_;
  
  if ( defined $field && validIDEntry($field) ) {
        my $parent_id = getID( $field );
        if ( !defined $family_tree_data->{people}{$parent_id} ) {  
          $family_tree_data->{people}{$parent_id} = Person->new($parent_id);            
          setName($family_tree_data->{people}{$parent_id}, $field);
        } 
        my $a_parent = $family_tree_data->{people}{$parent_id};
        push @{ $family_tree_data->{children}{$parent_id}}, $temp_person;        
        $temp_person->{$parent_type} = $a_parent;
        if(defined $a_parent->{gender} && $a_parent->{gender} != $parent_type_code ){
          print "Incorrent sex for ", $a_parent->full_name(), "!?!\n";
        }
        else {
          $a_parent->{gender} = $parent_type_code;
        }
      }
}
#
# return: 0, in case of file open error
sub createFamityTreeDataFromFile {
  my ($file_name, $picture_directory) = @_;				
  my $family_tree_data = FamilyTreeData->new();
  open DATAFILE, "< $file_name" or return 0;

  while (<DATAFILE>) {
    chomp;
    my @fields = split( /;/, $_ );
    if ( validIDEntry($fields[0]) ) {
      $family_tree_data->{people}{ getID( $fields[0] ) } = Person->new( getID( $fields[0] ) )
        if ( !defined $family_tree_data->{people}{ getID( $fields[0] ) } );
      my $temp_person = $family_tree_data->{people}{ getID( $fields[0] ) };
      setName($temp_person, $fields[0]);
      set_parent($fields[1], $family_tree_data, $temp_person, "father", 0);
      set_parent($fields[2], $family_tree_data, $temp_person, "mother", 1);
      $temp_person->{email} = $fields[3] if(validEmail($fields[3]));
      $temp_person->{homepage} = $fields[4] if(validURL($fields[4]));
      if ( defined $fields[5] ) {
        my ( $date_of_birth, $date_of_death ) = split( /-/, $fields[5] );
        $temp_person->{date_of_birth} = getDate($date_of_birth)
          if ( defined $date_of_birth && $date_of_birth ne "" );
        $temp_person->{date_of_death} = getDate($date_of_death)
          if ( defined $date_of_death && $date_of_death ne "");
      }
      $temp_person->{gender} = $fields[6]
        if( validGender($fields[6]));
    }
    else {
      print "First column of a row is not a valid ID. Row: @fields ignored!\n";
    }
  }
  close(DATAFILE);
  fill_up_pictures($family_tree_data, $picture_directory);
  return $family_tree_data;
}

#######################################################
# converts a name to a filename
# (converts spaces, converts case)
# CHANGE: we dont remove middle name: $newname =~ s/ .* / /;
#
sub getFileName {
  my ($id) = @_;
  $id =~ s/ /_/g;
  $id =~ tr/A-Z/a-z/;
#  $id =~ tr/[á,é,ó,ö,ő,ú,ü,ű,í,Á,É,Ó,Ö,Ő,Ú,Ü,Ű,Í]/
#            [a,e,o,o,o,u,u,u,i,a,e,o,o,o,u,u,u,i]/; #IT DOES NOT WORK?!?
            
# This works:        
  $id =~ s/á/a/g;
  $id =~ s/ä/a/g;
  $id =~ s/é/e/g;
  $id =~ s/ó/o/g;
  $id =~ s/ö/o/g;
  $id =~ s/ő/o/g;
  $id =~ s/ú/u/g;
  $id =~ s/ü/u/g;
  $id =~ s/ű/u/g;
  $id =~ s/í/i/g;
  $id =~ s/Á/a/g;
  $id =~ s/É/e/g;
  $id =~ s/Ó/o/g;
  $id =~ s/Ö/o/g;
  $id =~ s/Ő/o/g;
  $id =~ s/Ú/u/g;
  $id =~ s/Ü/u/g;
  $id =~ s/Ű/u/g;
  $id =~ s/Í/i/g;
  $id =~ s/ß/b/g;
  return $id;
}
1;

